import sys
import os
import AlteryxPythonSDK as Sdk
import requests
from requests import sessions
from requests.auth import HTTPProxyAuth
from urllib.parse import urlparse
from pypac import PACSession, get_pac

# global defines
_base_session = sessions.Session
_pac_session = PACSession()
_pac = _pac_session.get_pac()
_proxy_info = Sdk.ProxyInfo()

if _proxy_info.auth_required == True:
    _pac_session.proxy_auth = HTTPProxyAuth(_proxy_info.username,
                                            _proxy_info.password)

class AYXSession(_base_session):
    """Requests Session Inherentance Class

    This class is used as a substitute for requests.Session class, that once
    imported into a Python module will replace subsequent requests.Session class
    imports with this one that adds proxy support inherently.
    """
    def __init__(self):
        super(AYXSession, self).__init__()

    def _set_head_url(self, url):
        # try to access the URL so it goes through the proxy failover process
        # banned/failed proxies are stored in the resolver inside the session
        try:
            _pac_session.head(url)
        except: # ignore any exceptions here
            pass

    def _host_name(self, url):
        # parse incomming url string to scheme://hostname:port only
        _parsed = urlparse(url)
        return "{}://{}{}{}".format(_parsed.scheme, _parsed.hostname,
                                    ':' if _parsed.port else '', _parsed.port if _parsed.port else '')
        
    def set_proxy_for_url(self, url):
        if _pac:
            self._set_head_url(url)
            resolver = _pac_session._proxy_resolver
            self.proxies = resolver.get_proxy_for_requests(url)
            os.environ['HTTP_PROXY'] = self.proxies.get('http') or ''
            os.environ['HTTPS_PROXY'] = self.proxies.get('https') or ''
        else:
            self.proxies = _proxy_info.get_manual_proxy(url)
            if self.proxies:
                os.environ['HTTP_PROXY'] = self.proxies
                os.environ['HTTPS_PROXY'] = self.proxies
            else:
                os.environ['HTTP_PROXY'] = ''
                os.environ['HTTPS_PROXY'] = ''

    def send(self, request, **kwargs):
        self.set_proxy_for_url(self._host_name(request.url))
        proxies = self.proxies or {}
        kwargs['proxies'] = proxies
        return super(AYXSession, self).send(request, **kwargs)

# global sys module substitution for requests.sessions.Session
sys.modules['requests'].Session = AYXSession
sys.modules['requests.sessions'].Session = AYXSession
